'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var Cookies = _interopDefault(require('js-cookie'));
var sdkJs = require('@sajari/sdk-js');
var React = require('react');
var emotionTheming = require('emotion-theming');
var reactAriaLive = require('react-aria-live');
var memoize = _interopDefault(require('fast-memoize'));
var i18next = _interopDefault(require('i18next'));
var LngDetector = _interopDefault(require('i18next-browser-languagedetector'));
var emotion = require('emotion');
var styled = _interopDefault(require('react-emotion'));
var Downshift = _interopDefault(require('downshift'));
var AutosizeInput = _interopDefault(require('react-input-autosize'));
var chroma = _interopDefault(require('chroma-js'));
var ReactSelect = _interopDefault(require('react-select'));

var Listener = /** @class */ (function () {
    /**
     * Constructs a listener object.
     */
    function Listener() {
        this.listeners = [];
    }
    /**
     * Adds a callback to the listener.
     * Returns a function that will unregister the callback from the listener when called.
     * @param callback The callback to be registered.
     * @return The unregister function to remove the callback from the listener.
     */
    Listener.prototype.listen = function (callback) {
        var _this = this;
        this.listeners.push(callback);
        return function () { return _this.unlisten(callback); };
    };
    /**
     * Removes a callback from the listener.
     */
    Listener.prototype.unlisten = function (callback) {
        var index = this.listeners.indexOf(callback);
        if (index >= 0) {
            this.listeners.splice(index, 1);
        }
    };
    /**
     * Notify takes a function and calls it for every listener.
     * The listener is supplied as the first argument to the function.
     * @param {function(callback: Function)} fn Function to call each of the callbacks in the listener with.
     */
    Listener.prototype.notify = function (fn) {
        this.listeners.forEach(function (l) {
            try {
                fn(l);
            }
            catch (e) {
                // tslint:disable-next-line no-console
                if (console && console.error) {
                    // tslint:disable-next-line no-console
                    console.error(e);
                }
            }
        });
    };
    return Listener;
}());

var EVENT_SEARCH_SENT = "search-sent";
var EVENT_RESPONSE_UPDATED = "response-updated";
var EVENT_RESULT_CLICKED = "result-clicked";
var EVENT_VALUES_UPDATED = "values-changed";
var EVENT_TRACKING_RESET = "tracking-reset";
var EVENT_ANALYTICS_PAGE_CLOSED = "page-close-analytics";
var EVENT_ANALYTICS_BODY_RESET = "body-reset-analytics";
var EVENT_ANALYTICS_RESULT_CLICKED = "result-clicked-analytics";
var EVENT_SELECTION_UPDATED = "selection-updated";
var EVENT_OPTIONS_UPDATED = "options-updated";

var events = [
    EVENT_ANALYTICS_PAGE_CLOSED,
    EVENT_ANALYTICS_BODY_RESET,
    EVENT_ANALYTICS_RESULT_CLICKED
];
/**
 * Analytics is an adaptor which listens for events on Pipeline and
 * Tracking and re-emits them as analytics-based events.
 */
var Analytics = /** @class */ (function () {
    /**
     * Constructs an analytics object that operates on the specified pipeline.
     */
    function Analytics(pipeline, tracking) {
        var _a;
        var _this = this;
        /**
         * Runs before the page is closed/navigated away from. Can trigger a ga onPageClose call.
         */
        this.beforeunload = function () {
            if (_this.enabled && _this.body) {
                _this.listeners.get(EVENT_ANALYTICS_PAGE_CLOSED).notify(function (callback) {
                    callback(_this.body);
                });
                _this.enabled = false; // TODO(tbillington): unload -> disable!!
            }
        };
        /**
         * Resets the currently held parameters. Can trigger a ga onBodyReset call.
         */
        this.resetBody = function () {
            if (_this.enabled) {
                _this.listeners.get(EVENT_ANALYTICS_BODY_RESET).notify(function (callback) {
                    callback(_this.body);
                });
                _this.longestNonAutocompletedBody = "";
                _this.longestAutocompletedBody = "";
                _this.enabled = false;
            }
        };
        /**
         * Runs when the response has been updated. Updates the currently held search parameters.
         */
        this.responseUpdated = function (response) {
            if (response.isEmpty() || response.isError()) {
                return;
            }
            _this.enabled = true;
            var originalBody = response.getQueryValues().get(_this.bodyLabel) ||
                "";
            var responseBody = response.getValues().get(_this.bodyAutocompletedLabel) || originalBody;
            _this.body = responseBody;
            // Here we check the lengths of the non-autocompleted bodies.
            // We do this because while the user is backspacing their query
            // the new autocompleted body may be longer than their actual input,
            // but we want to record their input rather than a completion resulting
            // from them removing chars.
            if (originalBody.length >= _this.longestNonAutocompletedBody.length) {
                _this.longestNonAutocompletedBody = originalBody;
                _this.longestAutocompletedBody = responseBody;
            }
        };
        /**
         * Runs when a result has been clicked. Can trigger a ga onResultClicked call.
         */
        this.resultClicked = function () {
            if (_this.enabled && _this.body) {
                _this.listeners.get(EVENT_ANALYTICS_RESULT_CLICKED).notify(function (callback) {
                    callback(_this.body);
                });
                _this.longestNonAutocompletedBody = "";
                _this.longestAutocompletedBody = "";
                _this.enabled = false;
            }
        };
        this.enabled = false;
        this.body = "";
        this.pipeline = pipeline;
        this.tracking = tracking;
        this.listeners = new Map(Object.entries((_a = {},
            _a[EVENT_ANALYTICS_PAGE_CLOSED] = new Listener(),
            _a[EVENT_ANALYTICS_BODY_RESET] = new Listener(),
            _a[EVENT_ANALYTICS_RESULT_CLICKED] = new Listener(),
            _a)));
        // longest values are for sending the users last intended query on reset
        this.longestNonAutocompletedBody = "";
        this.longestAutocompletedBody = "";
        // default to working with website pipeline values
        this.bodyLabel = "q";
        this.bodyAutocompletedLabel = "q";
        window.addEventListener("beforeunload", this.beforeunload);
        this.pipeline.listen(EVENT_RESPONSE_UPDATED, this.responseUpdated);
        this.pipeline.listen(EVENT_RESULT_CLICKED, this.resultClicked);
        this.tracking.listen(EVENT_TRACKING_RESET, this.resetBody);
    }
    /**
     * Register a listener for a specific event.
     * @param event Event to listen for
     * @param callback Callback to run when the event happens.
     * @return The unregister function to remove the callback from the listener.
     */
    Analytics.prototype.listen = function (event, callback) {
        if (events.indexOf(event) === -1) {
            throw new Error("unknown event type \"" + event + "\"");
        }
        return this.listeners.get(event).listen(callback);
    };
    return Analytics;
}());

var GoogleAnalyticsObjects;
(function (GoogleAnalyticsObjects) {
    GoogleAnalyticsObjects["UniversalAnalytics"] = "_ua";
    GoogleAnalyticsObjects["AnalyticsJS"] = "ga";
    GoogleAnalyticsObjects["GTag"] = "gtag";
})(GoogleAnalyticsObjects || (GoogleAnalyticsObjects = {}));
var GoogleAnalytics = /** @class */ (function () {
    /**
     * Constructs a GoogleAnalytics object.
     * @param {Analytics} analytics The analytics object to attach to.
     * @param {string} [id=undefined] The name of the ga global object. Defaults to "ga" or "_ua" if one isn't supplied.
     * @param {string} [param="q"] The URL parameter to use to indicate a search. Default to "q".
     */
    function GoogleAnalytics(analytics, id, param) {
        if (param === void 0) { param = "q"; }
        var _this = this;
        this.unregisterFunctions = [];
        /**
         * Stops this object listening for events.
         */
        this.detatch = function () { return _this.unregisterFunctions.forEach(function (fn) { return fn(); }); };
        /**
         * Callback for when the body has been reset. Calls sendGAPageView.
         */
        this.onBodyReset = function (body) { return _this.sendGAPageView(body); };
        /**
         * Callback for when a result has been clicked. Calls sendGAPageView.
         */
        this.onResultClicked = function (body) { return _this.sendGAPageView(body); };
        /**
         * Callback for when the page has been closed. Calls sendGAPageView.
         */
        this.onPageClose = function (body) { return _this.sendGAPageView(body); };
        this.unregisterFunctions.push(analytics.listen(EVENT_ANALYTICS_PAGE_CLOSED, this.onPageClose));
        this.unregisterFunctions.push(analytics.listen(EVENT_ANALYTICS_BODY_RESET, this.onBodyReset));
        this.unregisterFunctions.push(analytics.listen(EVENT_ANALYTICS_RESULT_CLICKED, this.onResultClicked));
        if (id !== undefined) {
            this.id = id;
        }
        else if (isFunction(window[GoogleAnalyticsObjects.AnalyticsJS])) {
            this.id = GoogleAnalyticsObjects.AnalyticsJS;
        }
        else if (isFunction(window[GoogleAnalyticsObjects.UniversalAnalytics])) {
            this.id = GoogleAnalyticsObjects.UniversalAnalytics;
        }
        else if (isFunction(window[GoogleAnalyticsObjects.GTag])) {
            this.id = GoogleAnalyticsObjects.GTag;
        }
        else {
            this.id = null;
        }
        this.param = param;
    }
    /**
     * Sends a page view event if ga is found on the page and we're not in dev mode.
     */
    GoogleAnalytics.prototype.sendGAPageView = function (body) {
        var _a;
        // @ts-ignore: window is an object
        if (this.id && isFunction(window[this.id])) {
            // Merge the body in with the existing query params in the url
            var pageAddress = url.augmentUri(
            // Take only the portion of the url following the domain
            location.href.substring(location.origin.length), (_a = {}, _a[this.param] = body, _a));
            if (this.id === GoogleAnalyticsObjects.GTag) {
                window[this.id]("event", "page_view", {
                    page_location: pageAddress
                });
            }
            else {
                window[this.id]("send", "pageview", pageAddress);
            }
        }
    };
    return GoogleAnalytics;
}());
var url = {
    /**
     * Convert a query string in to an object
     */
    decodeUriArgs: function (queryStr) {
        var args = {};
        var a = queryStr.split("&");
        for (var i in a) {
            if (a.hasOwnProperty(i)) {
                var b = a[i].split("=");
                args[decodeURIComponent(b[0])] = decodeURIComponent(b[1]);
            }
        }
        return args;
    },
    /**
     * Convert an arguments object in to a query string
     */
    encodeUriArgs: function (args) {
        var queryParts = [];
        Object.keys(args).forEach(function (key) {
            return queryParts.push(encodeURIComponent(key) + "=" + encodeURIComponent(args[key]));
        });
        return queryParts.join("&");
    },
    /**
     * Merges query strings or objects into a single query string. Accepts a variable number of query string/objects
     * to merge. The latter overrides the former.
     */
    mergeQueryStr: function (first) {
        var _this = this;
        var rest = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            rest[_i - 1] = arguments[_i];
        }
        var args = typeof first === "string" ? this.decodeUriArgs(first) : first;
        rest.forEach(function (arg) {
            var next = typeof arg === "string" ? _this.decodeUriArgs(arg) : arg;
            Object.keys(next).forEach(function (prop) { return (args[prop] = next[prop]); });
        });
        return this.encodeUriArgs(args);
    },
    /**
     * Takes an existing URL and merges additional data into the query string
     */
    augmentUri: function (uri, args) {
        var m = /^([^?]+)\?(.+)+$/.exec(uri);
        if (m) {
            return m[1] + "?" + this.mergeQueryStr(m[2], args);
        }
        else {
            return uri + "?" + this.encodeUriArgs(args);
        }
    },
    /**
     * Get a parameter from the URL
     */
    getURLParameter: function (name) {
        var value = new RegExp("[?|&]" + name + "=" + "([^&;]+?)(&|#|;|$)").exec(location.search) || [undefined, ""];
        return (decodeURIComponent(value[1].replace(/\+/g, "%20")) || null);
    }
};
var isFunction = function (x) { return typeof x === "function"; };

var DebugAnalytics = /** @class */ (function () {
    /**
     *
     * @param {Analytics} analytics Analytics adaptor to listen for events on.
     */
    function DebugAnalytics(analytics) {
        this.pageClosed = function (bodyToSend) {
            // tslint:disable-next-line no-console
            console.log("DebugAnalytics: pageClosed, body:", bodyToSend);
        };
        this.bodyReset = function (bodyToSend) {
            // tslint:disable-next-line no-console
            console.log("DebugAnalytics: bodyReset, body:", bodyToSend);
        };
        this.resultClicked = function (bodyToSend) {
            // tslint:disable-next-line no-console
            console.log("DebugAnalytics: resultClicked, body:", bodyToSend);
        };
        analytics.listen(EVENT_ANALYTICS_PAGE_CLOSED, this.pageClosed);
        analytics.listen(EVENT_ANALYTICS_BODY_RESET, this.bodyReset);
        analytics.listen(EVENT_ANALYTICS_RESULT_CLICKED, this.resultClicked);
    }
    return DebugAnalytics;
}());

var Response = /** @class */ (function () {
    /**
     * Constructs a Response object.
     * @param error
     * @param queryValues
     * @param response
     * @param values
     */
    function Response(error, queryValues, response, values) {
        this.error = error;
        this.queryValues = queryValues;
        this.response = response;
        this.values = values;
    }
    /**
     * Is this response empty?
     */
    Response.prototype.isEmpty = function () {
        return (this.error === null &&
            this.response === undefined &&
            this.values === undefined &&
            this.queryValues === undefined);
    };
    /**
     * Is this response an error?
     */
    Response.prototype.isError = function () {
        return this.error !== null;
    };
    /**
     * The error associated with this response.
     */
    Response.prototype.getError = function () {
        return this.error;
    };
    /**
     * Return the query values used in the search which created this response.
     */
    Response.prototype.getQueryValues = function () {
        return this.queryValues;
    };
    /**
     * Returns the response, which includes results and aggregates etc.
     */
    Response.prototype.getResponse = function () {
        return this.response;
    };
    /**
     * Return the pipeline values returned by the search.
     */
    Response.prototype.getValues = function () {
        return this.values;
    };
    /**
     * Return results from the response.
     */
    Response.prototype.getResults = function () {
        return this.response !== undefined
            ? this.response.get("results")
            : undefined;
    };
    /**
     * Return the total number of results.
     */
    Response.prototype.getTotalResults = function () {
        return this.response !== undefined
            ? this.response.get("totalResults")
            : undefined;
    };
    /**
     * Return time from the response.
     */
    Response.prototype.getTime = function () {
        return this.response !== undefined
            ? this.response.get("time")
            : undefined;
    };
    /**
     * Return the aggregates in the response.
     */
    Response.prototype.getAggregates = function () {
        if (this.response === undefined) {
            return undefined;
        }
        var aggregates = this.response.get("aggregates");
        if (aggregates === undefined) {
            return undefined;
        }
        return aggregates;
    };
    return Response;
}());

var events$1 = [EVENT_TRACKING_RESET];
var Tracking = /** @class */ (function () {
    function Tracking() {
        var _a;
        this.clientTracking = null;
        this.listeners = new Map(Object.entries((_a = {},
            _a[EVENT_TRACKING_RESET] = new Listener(),
            _a)));
    }
    /**
     * Register a listener for a specific event.
     * @param event Event to listen for
     * @param callback Callback to run when the event happens.
     * @return The unregister function to remove the callback from the listener.
     */
    Tracking.prototype.listen = function (event, callback) {
        if (events$1.indexOf(event) === -1) {
            throw new Error("unknown event type \"" + event + "\"");
        }
        return this.listeners.get(event).listen(callback);
    };
    /**
     * Emits a tracking reset event to the tracking reset event listener.
     * @private
     */
    Tracking.prototype._emitTrackingReset = function (values) {
        this.listeners.get(EVENT_TRACKING_RESET).notify(function (listener) {
            listener(values);
        });
    };
    /**
     * Reset the tracking.
     * @param values Key-value pair parameters to use in the pipeline.
     */
    Tracking.prototype.reset = function (values) {
        throw new Error("method 'reset' unimplemented");
    };
    /**
     * Tracking returns the tracking data to be attached to the pipeline request.
     * @param values Key-value pair parameters to use in the pipeline.
     * @return Tracking values to be used in the search request.
     */
    Tracking.prototype.next = function (values) {
        throw new Error("method 'next' unimplemented");
    };
    return Tracking;
}());

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

// @ts-ignore: module missing defintion file
var getTrackingData = function () {
    var data = {};
    var ga = Cookies.get("_ga");
    if (ga) {
        data.ga = ga;
    }
    var sjID = Cookies.get("sjID");
    if (sjID) {
        data.sjID = sjID;
    }
    return data;
};

var ClickTracking = /** @class */ (function (_super) {
    __extends(ClickTracking, _super);
    /**
     * Construct a ClickTracking instance.
     *
     * @param field Field to use for click token generation.
     * @param qParam Value to use for full-text query param.
     */
    function ClickTracking(field, qParam) {
        if (field === void 0) { field = "url"; }
        if (qParam === void 0) { qParam = "q"; }
        var _this = _super.call(this) || this;
        _this.field = field;
        _this.qParam = qParam;
        _this.clientTracking = new sdkJs.InteractiveSession(qParam, new sdkJs.DefaultSession(sdkJs.TrackingType.Click, field, getTrackingData()));
        _this.prevQ = "";
        return _this;
    }
    /**
     * Reset the tracking.
     * @param values Key-value pair parameters to use in the pipeline.
     */
    ClickTracking.prototype.reset = function (values) {
        this.clientTracking.reset();
        if (values !== undefined) {
            this._emitTrackingReset(values);
        }
    };
    /**
     * Construct a tracking session to be used in a search.
     *
     * @param values Key-value pair parameters to use in the pipeline.
     */
    ClickTracking.prototype.next = function (values) {
        if (this.clientTracking === null) {
            throw new Error("clientTracking is null");
        }
        return this.clientTracking.next(values);
    };
    return ClickTracking;
}(Tracking));

var NoTracking = /** @class */ (function (_super) {
    __extends(NoTracking, _super);
    /**
     * Construct a NoTracking instance.
     */
    function NoTracking() {
        var _this = _super.call(this) || this;
        _this.clientTracking = new sdkJs.DefaultSession(sdkJs.TrackingType.None, "_id", getTrackingData());
        return _this;
    }
    /**
     * Reset the tracking.
     * @param values Key-value pair parameters to use in the pipeline.
     */
    NoTracking.prototype.reset = function (values) {
        this.clientTracking.reset();
        if (values !== undefined) {
            this._emitTrackingReset(values);
        }
    };
    /**
     * Construct a tracking session to be used in a search.
     */
    NoTracking.prototype.next = function (values) {
        if (this.clientTracking === null) {
            throw new Error("clientTracking is null");
        }
        return this.clientTracking.next(values);
    };
    return NoTracking;
}(Tracking));

var events$2 = [
    EVENT_SEARCH_SENT,
    EVENT_RESPONSE_UPDATED,
    EVENT_RESULT_CLICKED
];
var Pipeline = /** @class */ (function () {
    /**
     * Constructs a Pipeline object.
     * @param config Project, Collection config
     * @param name Name of the pipeline.
     * @param [tracking=ClickTracking()] Default tracking to use in searches.
     * @param [analyticsAdapters=GoogleAnalytics]
     */
    function Pipeline(config, name, tracking, analyticsAdapters) {
        if (tracking === void 0) { tracking = new NoTracking(); }
        if (analyticsAdapters === void 0) { analyticsAdapters = [GoogleAnalytics]; }
        var _this = this;
        this.response = new Response(null);
        var project = config.project, collection = config.collection, endpoint = config.endpoint;
        this.config = config;
        var opts = endpoint !== undefined ? [sdkJs.withEndpoint(endpoint)] : [];
        this.pipeline = new sdkJs.Client(project, collection, opts).pipeline(name);
        this.name = name;
        this.tracking = tracking;
        this.listeners = new Map([
            [EVENT_SEARCH_SENT, new Listener()],
            [EVENT_RESPONSE_UPDATED, new Listener()],
            [EVENT_RESULT_CLICKED, new Listener()]
        ]);
        this.searchCount = 0;
        this.response = new Response(null);
        this.analytics = new Analytics(this, this.tracking);
        analyticsAdapters.forEach(function (Adapter) {
            // tslint:disable-next-line
            new Adapter(_this.analytics);
        });
    }
    /**
     * Register a listener for a specific event.
     * @param event Event to listen for
     * @param callback Callback to run when the event happens.
     * @return The unregister function to remove the callback from the listener.
     */
    Pipeline.prototype.listen = function (event, callback) {
        if (events$2.indexOf(event) === -1) {
            throw new Error("unknown event type \"" + event + "\"");
        }
        return this.listeners.get(event).listen(callback);
    };
    /**
     * Emits a search event to the search event listener.
     * @private
     */
    Pipeline.prototype._emitSearchSent = function (values) {
        this.listeners.get(EVENT_SEARCH_SENT).notify(function (listener) {
            listener(values);
        });
    };
    /**
     * Emits a results event to the results event listener.
     * @private
     */
    Pipeline.prototype._emitResponseUpdated = function (response) {
        this.listeners.get(EVENT_RESPONSE_UPDATED).notify(function (listener) {
            listener(response);
        });
    };
    /**
     * Emits a result clicked event to the results clicked event listeners.
     * @param value Value to send to the listeners.
     */
    Pipeline.prototype.emitResultClicked = function (value) {
        this.listeners.get(EVENT_RESULT_CLICKED).notify(function (listener) {
            listener(value);
        });
    };
    /**
     * Perform a search.
     * @param values Key-value parameters to pass to the pipeline.
     */
    Pipeline.prototype.search = function (values) {
        var _this = this;
        this.searchCount++;
        var currentSearch = this.searchCount;
        this.pipeline.search(values, this.tracking, function (error, response, responseValues) {
            if (responseValues === void 0) { responseValues = {}; }
            if (currentSearch < _this.searchCount) {
                return;
            }
            _this.response = new Response(error ? error : null, new Map(Object.entries(values)), response !== undefined
                ? new Map(Object.entries(response))
                : undefined, new Map(Object.entries(responseValues)));
            // tslint:disable-next-line no-console
            if (error && console && console.error) {
                // tslint:disable-next-line no-console
                console.error(error);
            }
            _this._emitResponseUpdated(_this.response);
        });
        this._emitSearchSent(values);
    };
    /**
     * Clears the error, response, and response values from this object.
     * @param values Key-value pair parameters.
     */
    Pipeline.prototype.clearResponse = function (values) {
        this.tracking.next(values);
        this.searchCount++;
        this.response = new Response(null);
        this._emitResponseUpdated(this.response);
    };
    /**
     * The current response.
     */
    Pipeline.prototype.getResponse = function () {
        return this.response;
    };
    /**
     * The analytics adaptor connected to this pipeline.
     */
    Pipeline.prototype.getAnalytics = function () {
        return this.analytics;
    };
    return Pipeline;
}());

var Values = /** @class */ (function () {
    /**
     * Constructor for Values object.
     * @param values Initial values.
     */
    function Values(values) {
        if (values === void 0) { values = {}; }
        this.listeners = new Map([[EVENT_VALUES_UPDATED, new Listener()]]);
        this.values = new Map(Object.entries(values));
    }
    /**
     * Register a listener for a specific event.
     * @param event Event to listen for
     * @param callback Callback to run when the event happens.
     */
    Values.prototype.listen = function (event, callback) {
        if (event !== EVENT_VALUES_UPDATED) {
            throw new Error("unknown event type \"" + event + "\"");
        }
        return this.listeners.get(event).listen(callback);
    };
    /**
     * Merge values into the value map.
     *
     * Set a value to undefined to remove it.
     */
    Values.prototype.set = function (values) {
        this._set(values);
        this._emitUpdated(values);
    };
    /**
     * get returns the current values.
     */
    Values.prototype.get = function () {
        var values = {};
        this.values.forEach(function (value, key) {
            if (typeof value === "function") {
                values[key] = value();
            }
            else if (Array.isArray(value)) {
                values[key] = value.join(",");
            }
            else {
                values[key] = String(value);
            }
        });
        return values;
    };
    /**
     * Emits an event to notify listener that the values have been updated.
     *
     * @private
     */
    Values.prototype._emitUpdated = function (changes) {
        var _this = this;
        this.listeners.get(EVENT_VALUES_UPDATED).notify(function (listener) {
            return listener(changes, function (values) { return _this._set(values); });
        });
    };
    /**
     * Sets values without triggering an event, internal use only.
     */
    Values.prototype._set = function (values) {
        var _this = this;
        Object.keys(values).forEach(function (key) {
            if (values[key] === undefined) {
                _this.values.delete(key);
            }
            else {
                _this.values.set(key, values[key]);
            }
        });
    };
    return Values;
}());

var events$3 = [EVENT_SELECTION_UPDATED, EVENT_OPTIONS_UPDATED];
/**
 * Filter is a helper class for building filters from UI components.
 */
var Filter = /** @class */ (function () {
    /**
     * Constructs an instance of Filter.
     *
     * @example
     * const filter = new Filter({});
     */
    function Filter(options, // Dictionary of name -> filter pairs
    initial, // List of initially selected items
    multi, // Multiple selections allowed?
    joinOperator // Join operator used if multi = true
    ) {
        var _a;
        if (initial === void 0) { initial = []; }
        if (multi === void 0) { multi = false; }
        if (joinOperator === void 0) { joinOperator = "OR"; }
        if (typeof initial === "string") {
            initial = [initial];
        }
        /** @private */
        this.current = initial;
        /** @private */
        this.options = options;
        /** @private */
        this.multi = multi;
        /** @private */
        this.joinOperator = joinOperator;
        /** @private */
        this.listeners = (_a = {},
            _a[EVENT_SELECTION_UPDATED] = new Listener(),
            _a[EVENT_OPTIONS_UPDATED] = new Listener(),
            _a);
    }
    /**
     * Register a listener for a specific event.
     */
    Filter.prototype.listen = function (event, callback) {
        if (events$3.indexOf(event) === -1) {
            throw new Error("unknown event type \"" + event + "\"");
        }
        return this.listeners[event].listen(callback);
    };
    /**
     * Set the state of the filter.
     */
    Filter.prototype.set = function (name, on) {
        if (this.multi) {
            this._setMulti(name, on);
            return;
        }
        // if on add to current
        if (on) {
            this.current = [name];
        }
        else {
            // clear current
            this.current = [];
        }
        this._emitSelectionUpdated();
    };
    /**
     * returns whether the filter is set or not.
     */
    Filter.prototype.isSet = function (name) {
        return this.current.indexOf(name) !== -1;
    };
    /**
     * Merge options into the filter options.
     *
     * Set an option to null to remove it.
     */
    Filter.prototype.setOptions = function (options) {
        var _this = this;
        Object.keys(options).forEach(function (k) {
            if (options[k] === null) {
                delete _this.options[k];
                _this.current = _this.current.filter(function (n) { return n !== k; });
            }
            else {
                _this.options[k] = options[k];
            }
        });
        this._emitOptionsUpdated();
    };
    /**
     * Get all the options defined in this filter.
     */
    Filter.prototype.getOptions = function () {
        var fields = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            fields[_i] = arguments[_i];
        }
        return this.options;
    };
    /**
     * Get the current selection in this filter.
     */
    Filter.prototype.get = function () {
        var fields = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            fields[_i] = arguments[_i];
        }
        return this.current;
    };
    /**
     * Builds up the filter string from the current filter and it's children.
     */
    Filter.prototype.filter = function () {
        var _this = this;
        var filters = this.current
            .map(function (c) {
            var f = _this.options[c];
            if (typeof f === "function") {
                f = f();
            }
            if (f !== "") {
                f = "(" + f + ")";
            }
            return f;
        })
            .filter(Boolean);
        switch (filters.length) {
            case 0:
                return "";
            case 1:
                return filters[0];
            default:
                return filters.join(" " + this.joinOperator + " ");
        }
    };
    /**
     * Emits a selection updated event to the selection updated listener.
     * @private
     */
    Filter.prototype._emitSelectionUpdated = function () {
        this.listeners[EVENT_SELECTION_UPDATED].notify(function (listener) {
            listener();
        });
    };
    /**
     * Emits an options updated event to the options updated listener.
     * @private
     */
    Filter.prototype._emitOptionsUpdated = function () {
        this.listeners[EVENT_OPTIONS_UPDATED].notify(function (listener) {
            listener();
        });
    };
    /** @private */
    Filter.prototype._setMulti = function (name, on) {
        if (on && this.current.indexOf(name) === -1) {
            this.current = this.current.concat(name);
        }
        else {
            this.current = this.current.filter(function (n) { return n !== name; });
        }
        this._emitSelectionUpdated();
    };
    return Filter;
}());
/**
 * CombineFilters is a helper for combining multiple Filter instances
 * into one.
 *
 * Whenever any of the combined filters are updated, the events are
 * propagated up to the returned "root" filter.
 *
 * @param filters Array of filters to combine.
 * @param  [operator="AND"] Operator to apply between them ("AND" | "OR").
 * @return The resulting Filter.
 */
var CombineFilters = function (filters, operator) {
    if (operator === void 0) { operator = "AND"; }
    var opts = {};
    var count = 1;
    var on = [];
    filters.forEach(function (f) {
        opts["" + count] = function () { return f.filter(); };
        on = on.concat(["" + count]);
        count++;
    });
    var combFilter = new Filter(opts, on, true, operator);
    filters.forEach(function (f) {
        f.listen(EVENT_SELECTION_UPDATED, function () {
            // @ts-ignore
            combFilter._emitSelectionUpdated();
        });
        f.listen(EVENT_OPTIONS_UPDATED, function () {
            // @ts-ignore
            combFilter._emitOptionsUpdated();
        });
    });
    return combFilter;
};

var PipelineContext = React.createContext({});

var Consumer = PipelineContext.Consumer;

var defaultConfig = {
    qParam: "q",
    qOverrideParam: "q.override",
    qSuggestionsParam: "q.suggestions",
    maxSuggestions: 5,
    resultsPerPageParam: "resultsPerPage",
    pageParam: "page"
};

var defaultState = {
    response: null,
    query: "",
    completion: "",
    suggestions: [],
    config: defaultConfig
};
var Provider = /** @class */ (function (_super) {
    __extends(Provider, _super);
    function Provider() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            search: defaultState,
            instant: defaultState
        };
        _this.unregisterFunctions = [];
        _this.getContext = function (state) {
            return (__assign({}, state, { search: __assign({}, state.search, { search: _this.search("search"), clear: _this.clear("search") }), instant: __assign({}, state.instant, { search: _this.search("instant"), clear: _this.clear("instant") }), resultClicked: _this.handleResultClicked, paginate: _this.handlePaginate }));
        };
        _this.search = function (key) { return function (query, override) {
            if (override === void 0) { override = false; }
            var _a;
            var _b = _this.props[key] || _this.instant, pipeline = _b.pipeline, values = _b.values;
            var config = _this.state[key].config;
            var text = (_a = {},
                _a[config.qParam] = query,
                _a[config.qOverrideParam] = undefined,
                _a);
            if (override) {
                text[config.qOverrideParam] = "true";
            }
            values.set(text);
            if (text[config.qParam]) {
                pipeline.search(values.get());
            }
            else {
                pipeline.clearResponse(values.get());
            }
        }; };
        _this.clear = function (key) { return function (vals) {
            var _a = _this.props[key] || _this.instant, pipeline = _a.pipeline, values = _a.values;
            if (vals !== undefined) {
                values.set(vals);
            }
            pipeline.clearResponse(values.get());
        }; };
        _this.handleResultClicked = function (url) {
            return _this.props.search.pipeline.emitResultClicked(url);
        };
        _this.handlePaginate = function (page) {
            var search = _this.props.search;
            var pipeline = search.pipeline, values = search.values;
            values.set({ page: String(page) });
            pipeline.search(values.get());
        };
        return _this;
    }
    Provider.prototype.componentDidMount = function () {
        var _this = this;
        var _a = this.props, search = _a.search, instant = _a.instant, searchOnLoad = _a.searchOnLoad;
        var mergedConfig = __assign({}, defaultConfig, search.config);
        this.setState(function (state) {
            return (__assign({}, state, { search: __assign({}, state.search, { response: search.pipeline.getResponse(), query: search.values.get()[mergedConfig.qParam] || "", config: mergedConfig }), instant: __assign({}, state.instant, { config: __assign({}, defaultConfig, search.config) }) }));
        });
        this.unregisterFunctions.push(search.pipeline.listen(EVENT_RESPONSE_UPDATED, function (response) {
            return _this.setState(function (state) { return (__assign({}, state, { search: __assign({}, state.search, { response: response }, responseUpdatedListener(search.values, state.search.config, response)) })); });
        }));
        this.unregisterFunctions.push(search.values.listen(EVENT_VALUES_UPDATED, function () {
            return _this.setState(function (state) { return (__assign({}, state, { search: __assign({}, state.search, valuesUpdatedListener(search.values, search.pipeline, state.search.config)) })); });
        }));
        this.instant = instant;
        if (this.instant === undefined) {
            var _b = search.pipeline.config, project = _b.project, collection = _b.collection, endpoint = _b.endpoint;
            this.instant = {
                pipeline: new Pipeline({ project: project, collection: collection, endpoint: endpoint }, "autocomplete", new NoTracking()),
                values: new Values()
            };
        }
        this.unregisterFunctions.push(this.instant.pipeline.listen(EVENT_RESPONSE_UPDATED, function (response) {
            return _this.setState(function (state) { return (__assign({}, state, { instant: __assign({}, state.instant, { response: response }, responseUpdatedListener(_this.instant.values, state.instant.config, response)) })); });
        }));
        this.unregisterFunctions.push(this.instant.values.listen(EVENT_VALUES_UPDATED, function () {
            return _this.setState(function (state) { return (__assign({}, state, { instant: __assign({}, state.instant, valuesUpdatedListener(_this.instant.values, _this.instant.pipeline, state.instant.config)) })); });
        }));
        if (searchOnLoad) {
            search.pipeline.search(search.values.get());
        }
    };
    Provider.prototype.componentWillUnmount = function () {
        this.unregisterFunctions.forEach(function (fn) { return fn(); });
        this.unregisterFunctions = [];
    };
    Provider.prototype.render = function () {
        var children = this.props.children;
        var value = this.getContext(this.state);
        return (React.createElement(PipelineContext.Provider, { value: value }, children));
    };
    return Provider;
}(React.PureComponent));
var responseUpdatedListener = function (values, config, response) {
    var query = values.get()[config.qParam] || "";
    var responseValues = response.getValues();
    return updateState(query, responseValues, config);
};
var valuesUpdatedListener = function (values, pipeline, config) {
    var query = values.get()[config.qParam] || "";
    var responseValues = pipeline.getResponse().getValues();
    return updateState(query, responseValues, config);
};
var updateState = function (query, responseValues, config) {
    var completion = query && responseValues ? responseValues.get(config.qParam) || "" : "";
    var suggestions = [];
    if (responseValues) {
        suggestions = (responseValues.get("q.suggestions") || "")
            .split(",")
            .filter(function (s) { return s.length > 0; })
            .slice(0, config.maxSuggestions);
    }
    return {
        completion: completion,
        query: query,
        suggestions: suggestions
    };
};

var Provider$1 = /** @class */ (function (_super) {
    __extends(Provider$$1, _super);
    function Provider$$1() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Provider$$1.prototype.render = function () {
        var _a = this.props, search = _a.search, instant = _a.instant, theme = _a.theme, searchOnLoad = _a.searchOnLoad, children = _a.children;
        return (React.createElement(Provider, { search: search, instant: instant, searchOnLoad: searchOnLoad },
            React.createElement(reactAriaLive.LiveAnnouncer, null,
                React.createElement(emotionTheming.ThemeProvider, { theme: theme || {} },
                    React.createElement(React.Fragment, null, children)))));
    };
    return Provider$$1;
}(React.PureComponent));

var Consumer$1 = /** @class */ (function (_super) {
    __extends(Consumer$$1, _super);
    function Consumer$$1() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Consumer$$1.prototype.render = function () {
        var children = this.props.children;
        if (typeof children !== "function") {
            // https://reactpatterns.com/#function-as-children
            throw new Error("Consumer requires a function as children");
        }
        return (React.createElement(Consumer, null, function (searchValue) { return children(__assign({}, searchValue)); }));
    };
    return Consumer$$1;
}(React.Component));

var Context = React.createContext({});

var FilterProvider = /** @class */ (function (_super) {
    __extends(FilterProvider, _super);
    function FilterProvider() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { options: {}, selected: [] };
        _this.unregisterFuntions = [];
        _this.getContext = memoize(function (state) { return (__assign({}, state, { set: function (name, value) { return _this.props.filter.set(name, value); } })); });
        return _this;
    }
    FilterProvider.prototype.componentDidMount = function () {
        var _this = this;
        var filter = this.props.filter;
        this.unregisterFuntions.push(filter.listen(EVENT_OPTIONS_UPDATED, function () {
            return _this.setState(function (state) { return (__assign({}, state, { options: filter.getOptions() })); });
        }));
        this.unregisterFuntions.push(filter.listen(EVENT_SELECTION_UPDATED, function () {
            return _this.setState(function (state) { return (__assign({}, state, { selected: filter.get() })); });
        }));
        this.setState(function (state) { return (__assign({}, state, { options: filter.getOptions(), selected: filter.get() })); });
    };
    FilterProvider.prototype.componentWillUnmount = function () {
        this.unregisterFuntions.forEach(function (fn) { return fn(); });
    };
    FilterProvider.prototype.render = function () {
        var children = this.props.children;
        var value = this.getContext(this.state);
        return React.createElement(Context.Provider, { value: value }, children);
    };
    return FilterProvider;
}(React.PureComponent));

var FilterConsumer = Context.Consumer;

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 *  strict
 * @format
 */

/**
 * Traverses properties on objects and arrays. If an intermediate property is
 * either null or undefined, it is instead returned. The purpose of this method
 * is to simplify extracting properties from a chain of maybe-typed properties.
 *
 * === EXAMPLE ===
 *
 * Consider the following type:
 *
 *   const props: {
 *     user: ?{
 *       name: string,
 *       friends: ?Array<User>,
 *     }
 *   };
 *
 * Getting to the friends of my first friend would resemble:
 *
 *   props.user &&
 *   props.user.friends &&
 *   props.user.friends[0] &&
 *   props.user.friends[0].friends
 *
 * Instead, `idx` allows us to safely write:
 *
 *   idx(props, _ => _.user.friends[0].friends)
 *
 * The second argument must be a function that returns one or more nested member
 * expressions. Any other expression has undefined behavior.
 *
 * === NOTE ===
 *
 * The code below exists for the purpose of illustrating expected behavior and
 * is not meant to be executed. The `idx` function is used in conjunction with a
 * Babel transform that replaces it with better performing code:
 *
 *   props.user == null ? props.user :
 *   props.user.friends == null ? props.user.friends :
 *   props.user.friends[0] == null ? props.user.friends[0] :
 *   props.user.friends[0].friends
 *
 * All this machinery exists due to the fact that an existential operator does
 * not currently exist in JavaScript.
 */

function idx(input, accessor) {
  try {
    return accessor(input);
  } catch (error) {
    if (error instanceof TypeError) {
      if (nullPattern.test(error)) {
        return null;
      } else if (undefinedPattern.test(error)) {
        return undefined;
      }
    }
    throw error;
  }
}

/**
 * Some actual error messages for null:
 *
 * TypeError: Cannot read property 'bar' of null
 * TypeError: Cannot convert null value to object
 * TypeError: foo is null
 * TypeError: null has no properties
 * TypeError: null is not an object (evaluating 'foo.bar')
 * TypeError: null is not an object (evaluating '(" undefined ", null).bar')
 */
var nullPattern = /^null | null$|^[^(]* null /i;
var undefinedPattern = /^undefined | undefined$|^[^(]* undefined /i;

idx.default = idx;
var idx_1 = idx;

var i18n = i18next.use(LngDetector).init({
    resources: {
        en: {
            errors: {
                authorization: "Authorization for this request failed. Check your credentials.",
                connection: "Please check your network connection.",
                parseResponse: "Unable to parse response."
            },
            summary: {
                page: "Page {{pageNumber}} of",
                resultsFor: "results for",
                searchInsteadFor: "search instead for"
            }
        }
    },
    fallbackLng: "en"
});

var Image = /** @class */ (function (_super) {
    __extends(Image, _super);
    function Image() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { error: false };
        _this.onError = function () { return _this.setState(function (state) { return (__assign({}, state, { error: true })); }); };
        return _this;
    }
    Image.getDerivedStateFromProps = function (nextProps, prevState) {
        var src = nextProps.src;
        if (src === undefined) {
            return { error: true };
        }
        return prevState;
    };
    Image.prototype.render = function () {
        var _a = this.props, alt = _a.alt, src = _a.src;
        var error = this.state.error;
        return error ? (React.createElement("div", { className: emotion.cx("sj-results__result__image", this.props.className, emotion.css({ width: 90, paddingRight: "1em" })) })) : (React.createElement("img", { className: emotion.cx("sj-results__result__image", this.props.className, emotion.css({ maxWidth: 90, maxHeight: 90, paddingRight: "1em" })), onError: this.onError, src: src, alt: alt }));
    };
    return Image;
}(React.PureComponent));

var TokenLink = /** @class */ (function (_super) {
    __extends(TokenLink, _super);
    function TokenLink() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { clicked: false };
        _this.click = function (_) {
            var _a = _this.props, url = _a.url, resultClicked = _a.resultClicked;
            resultClicked(url);
            _this.setState(function (state) { return (__assign({}, state, { clicked: true })); });
        };
        return _this;
    }
    TokenLink.prototype.render = function () {
        var _a = this.props, token = _a.token, url = _a.url, text = _a.text, _b = _a.styles, resultClicked = _a.resultClicked, children = _a.children, rest = __rest(_a, ["token", "url", "text", "styles", "resultClicked", "children"]);
        var decodedText;
        try {
            decodedText = decodeURI(url);
        }
        catch (e) {
            // encoded url was malformed, fall back to displaying the encoded text
            decodedText = url;
        }
        // if the text prop is provided, use that instead of the decoded URI
        decodedText = text ? text : decodedText;
        var renderChildren = children;
        if (renderChildren === undefined) {
            renderChildren = decodedText;
        }
        var className = emotion.cx(this.props.className, emotion.css(linkStyles));
        return (React.createElement("a", __assign({ href: this.state.clicked && token ? token : url }, rest, { className: className, onMouseDown: this.click }), renderChildren));
    };
    return TokenLink;
}(React.PureComponent));
var linkStyles = {
    textDecoration: "none",
    "&:hover": {
        cursor: "pointer",
        textDecoration: "underline"
    }
};

var Result = /** @class */ (function (_super) {
    __extends(Result, _super);
    function Result() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Result.prototype.render = function () {
        var _a = this.props, token = _a.token, values = _a.values, resultClicked = _a.resultClicked, _b = _a.showImage, showImage = _b === void 0 ? false : _b, theme = _a.theme;
        var styles = this.props.styles;
        if (styles === null || styles === undefined) {
            styles = {};
        }
        var title = values.title;
        var description = values.description;
        var url = values.url;
        var img = values.image;
        var classNames = {
            container: emotion.cx("sj-results__result", this.props.className, emotion.css(resultStyles.container), showImage && emotion.css({ flexDirection: "row" }), styles && styles.container && emotion.css(styles.container)),
            img: emotion.css({ display: "inline-block" }),
            textContainer: emotion.cx("sj-result__text", emotion.css({ minWidth: 0 })),
            title: emotion.cx("sj-results__result__title", emotion.css(resultStyles.title), styles && styles.title && emotion.css(styles.title), themeColor(theme), emotion.css({ "&:hover": themeColor(theme) })),
            description: emotion.cx("sj-results__result__description", emotion.css(resultStyles.description), styles && styles.description && emotion.css(styles.description)),
            url: emotion.cx("sj-results__result__link", emotion.css(resultStyles.link), styles && styles.url && emotion.css(styles.url), emotion.css({ "&:hover": themeColor(theme) }))
        };
        return (React.createElement("div", { className: classNames.container },
            showImage && (React.createElement(Image, { className: classNames.img, src: img, alt: "" })),
            React.createElement("div", { className: classNames.textContainer },
                React.createElement(TokenLink, { url: url, token: token, resultClicked: resultClicked, className: classNames.title },
                    React.createElement("h3", null, title)),
                React.createElement("p", { className: classNames.description }, description),
                React.createElement(TokenLink, { url: url, token: token, resultClicked: resultClicked, className: classNames.url }, url))));
    };
    return Result;
}(React.Component));
var Result$1 = emotionTheming.withTheme(Result);
var resultStyles = {
    container: {
        display: "flex",
        flexDirection: "column",
        minWidth: 0
    },
    title: {
        display: "inline-block",
        width: "100%",
        color: "#383b48",
        "&:hover": {
            color: "#383b48"
        },
        "& > h3": {
            fontSize: "1.1em",
            lineHeight: 1.1,
            fontWeight: 400,
            marginBottom: 0,
            marginTop: 0,
            overflow: "hidden",
            textOverflow: "ellipsis",
            whiteSpace: "nowrap"
        }
    },
    description: {
        fontSize: "0.85em",
        lineHeight: 1.69,
        marginBottom: 0,
        marginTop: 0,
        overflowWrap: "break-word",
        wordWrap: "break-word"
    },
    link: {
        display: "inline-block",
        width: "100%",
        fontSize: "0.75em",
        color: "#888991",
        margin: 0,
        overflow: "hidden",
        textOverflow: "ellipsis",
        whiteSpace: "nowrap",
        "&:hover": {
            color: "#22232b"
        }
    }
};
function themeColor(theme) {
    // @ts-ignore: idx
    return emotion.css({ color: idx_1(theme, function (_) { return _.colors.brand.primary; }) || "inherit" });
}

var override = function (props) { return (props.styles ? props.styles : {}); };

var Error$1 = styled("div")({ color: "red" }, override);
var Container = styled("ol")({
    display: "flex",
    flexWrap: "wrap",
    listStyle: "none",
    margin: 0,
    padding: 0,
    width: "100%"
}, override);
var gridItem = emotion.css({
    boxSizing: "border-box",
    padding: "0.5em",
    width: "25%"
});
var ResultItem = styled("li")({
    marginBottom: "1.5em",
    width: "100%"
}, function (_a) {
    var theme = _a.theme;
    // @ts-ignore: idx
    return (idx_1(theme, function (_) { return _.layout.type; }) || "list") === "grid" ? gridItem : {};
}, override);

var STATUS_UNAUTHORISED = 403;
var ResultsContainer = function (_a) {
    var children = _a.children, fields = _a.fields;
    return (React.createElement(Consumer$1, null, function (_a) {
        var response = _a.search.response, resultClicked = _a.resultClicked;
        if (response === null || response === undefined || response.isEmpty()) {
            return null;
        }
        if (response.isError()) {
            var err = response.getError();
            if (err.httpStatusCode === STATUS_UNAUTHORISED) {
                err.message = i18n.t("errors:authorization");
            }
            if (err.transportErrorCode === sdkJs.TransportError.Connection) {
                err.message = i18n.t("errors:connection");
            }
            else if (err.transportErrorCode === sdkJs.TransportError.ParseResponse) {
                err.message = i18n.t("errors:parseResponse");
            }
            return children({ error: err });
        }
        var results = response !== undefined ? response.getResults() || [] : [];
        if (results.length < 1) {
            return children({ error: null, results: [] });
        }
        var res = results.map(function (result, index) {
            var key = result.values._id ||
                ("" + index + result.values.url);
            var token = result.token && result.token.click;
            // tslint:disable:object-literal-sort-keys
            var values = __assign({}, result.values, { 
                // @ts-ignore: idx
                title: result.values[idx_1(fields, function (_) { return _.title; }) || "title"], description: result.values[
                // @ts-ignore: idx
                idx_1(fields, function (_) { return _.description; }) || "description"], 
                // @ts-ignore: idx
                url: result.values[idx_1(fields, function (_) { return _.url; }) || "url"], 
                // @ts-ignore: idx
                image: result.values[idx_1(fields, function (_) { return _.image; }) || "image"] });
            // tslint:enable:object-literal-sort-keys
            return {
                key: key,
                resultClicked: resultClicked,
                token: token,
                values: values,
                indexScore: result.indexScore,
                score: result.score
            };
        });
        return children({ error: null, results: res });
    }));
};
var Results = /** @class */ (function (_super) {
    __extends(Results, _super);
    function Results() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Results.prototype.render = function () {
        var _a = this.props, _b = _a.ResultRenderer, ResultRenderer = _b === void 0 ? Result$1 : _b, showImages = _a.showImages, fields = _a.fields, _c = _a.styles, styles = _c === void 0 ? {} : _c;
        return (React.createElement(ResultsContainer, { fields: fields }, function (_a) {
            var error = _a.error, results = _a.results;
            if (error) {
                return React.createElement(Error$1, null, error.message);
            }
            if (results === undefined) {
                return null;
            }
            return results.length > 0 ? (React.createElement(Container, { styles: idx_1(styles, function (_) { return _.container; }) }, results.map(function (_a) {
                var key = _a.key, result = __rest(_a, ["key"]);
                return (React.createElement(ResultItem, { key: key, styles: idx_1(styles, function (_) { return _.item; }) },
                    React.createElement(ResultRenderer, __assign({}, result, { showImage: showImages, styles: idx_1(styles, function (_) { return _.result; }) }))));
            }))) : null;
        }));
    };
    return Results;
}(React.Component));

var formatQueryTime = function (time) {
    if (!time) {
        return "";
    }
    var strTime = String(time);
    var splitNum = strTime.split(".");
    if (splitNum.length === 1 || parseInt(splitNum[1], 10) === 0) {
        return parseInt(splitNum[0], 10) + "s";
    }
    var numZeros = /0*/.exec(splitNum[1])[0].length;
    var fractional = splitNum[1].slice(0, numZeros + 1);
    return splitNum[0] + "." + fractional + "s";
};

var Container$1 = styled("div")({
    color: "#808080",
    fontSize: "small",
    marginBottom: "1.5em"
}, override);
var Emphasis = styled("strong")({ color: "#808080" }, function (_a) {
    var theme = _a.theme;
    return ({
        // @ts-ignore: idx
        color: idx_1(theme, function (_) { return _.colors.brandPrimary; }) || "#808080"
    });
}, override);
var OverrideContainer = styled("span")(override);

var Summary = /** @class */ (function (_super) {
    __extends(Summary, _super);
    function Summary() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Summary.prototype.render = function () {
        var _this = this;
        var _a = this.props, _b = _a.showQueryTime, showQueryTime = _b === void 0 ? true : _b, _c = _a.showQueryOverride, showQueryOverride = _c === void 0 ? true : _c, _d = _a.styles, styles = _d === void 0 ? {} : _d;
        return (React.createElement(Consumer$1, null, function (_a) {
            var _b = _a.search, response = _b.response, query = _b.query, config = _b.config, search = _b.search;
            if (response === null ||
                response === undefined ||
                response.isEmpty() ||
                response.isError()) {
                return null;
            }
            var responseValues = response.getValues();
            var queryValues = response.getQueryValues();
            var text = responseValues.get(config.qParam) || query;
            var page = parseInt(queryValues.get(config.pageParam), 10);
            var pageNumber = page && page > 1
                ? i18n.t("summary:page", { replace: { pageNumber: page } })
                : "";
            var totalResults = response.getTotalResults().toLocaleString();
            var responseTime = formatQueryTime(response.getTime());
            var ariaMessage = pageNumber + " " + totalResults + " " + i18n.t("summary:resultsFor") + " \"" + text + "\"";
            return (React.createElement(Container$1, { className: emotion.cx("sj-summary", _this.props.className), styles: idx_1(styles, function (_) { return _.container; }) },
                React.createElement(reactAriaLive.LiveMessage, { message: ariaMessage, "aria-live": "polite" }),
                React.createElement("span", { className: "sj-summary__results-text" }, pageNumber + " " + totalResults + " " + i18n.t("summary:resultsFor") + " ",
                    "\"",
                    React.createElement(Emphasis, { className: "sj-summary__search-term", styles: styles.searchTerm }, text),
                    "\"",
                    " "),
                showQueryTime && (React.createElement("span", { className: "sj-summary__query-time" }, "(" + responseTime + ") ")),
                showQueryOverride && (React.createElement(Override, { className: "sj-summary__query-override", responseQuery: responseValues.get(config.qParam), query: query, search: search, styles: idx_1(styles, function (_) { return _.override; }) }))));
        }));
    };
    return Summary;
}(React.Component));
var Override = function (_a) {
    var className = _a.className, responseQuery = _a.responseQuery, query = _a.query, search = _a.search, styles = _a.styles;
    if (!responseQuery || responseQuery.toLowerCase() === query.toLowerCase()) {
        return null;
    }
    if (styles === null || styles === undefined) {
        styles = {};
    }
    return (React.createElement(OverrideContainer, { className: className, styles: idx_1(styles, function (_) { return _.container; }) }, i18n.t("summary:searchInsteadFor") + " ",
        React.createElement("a", { onClick: click({ search: search, query: query }), href: "" }, query)));
};
var click = function (_a) {
    var search = _a.search, query = _a.query;
    return function (event) {
        event.preventDefault();
        search(query, true);
    };
};

var Dropdown = /** @class */ (function (_super) {
    __extends(Dropdown, _super);
    function Dropdown() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Dropdown.prototype.render = function () {
        var _a = this.props, downshift = _a.downshift, className = _a.className, styles = _a.styles, children = _a.children;
        return (React.createElement("div", { className: container },
            React.createElement("div", { className: content },
                React.createElement("ul", __assign({}, downshift.getMenuProps({
                    className: emotion.cx(className, listStyles, downshift.isOpen && listIsOpen, styles && emotion.css(styles))
                }), { "aria-labelledby": undefined }), children))));
    };
    return Dropdown;
}(React.Component));
// 10 million seems like a big enough z-index to be able to be above any
// potential content that it is overlaying
var DropdownZIndex = 10000000;
var container = emotion.css({
    boxSizing: "border-box",
    position: "relative",
    width: "100%",
    zIndex: DropdownZIndex
});
var content = emotion.css({
    boxSizing: "border-box",
    position: "absolute",
    width: "100%"
});
var listStyles = emotion.css({
    marginTop: 0,
    marginBottom: 0,
    paddingLeft: 0,
    backgroundColor: "#fff"
});
var listIsOpen = emotion.css({
    boxShadow: "0 3px 8px 0 rgba(0,0,0,0.16), 0 0 0 1px rgba(0,0,0,0.08)",
    boxSizing: "border-box",
    cursor: "default"
});

var Results$1 = /** @class */ (function (_super) {
    __extends(Results$$1, _super);
    function Results$$1() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Results$$1.prototype.render = function () {
        var _a = this.props, downshift = _a.downshift, showSummary = _a.showSummary, ResultRenderer = _a.ResultRenderer;
        return (React.createElement(ResultsContainer, null, function (_a) {
            var error = _a.error, results = _a.results;
            if (error) {
                return error.message;
            }
            if (results === undefined) {
                return "no results";
            }
            return (React.createElement(Dropdown, { downshift: downshift, className: "sj-input__results" }, downshift.isOpen &&
                downshift.inputValue !== "" && (React.createElement(React.Fragment, null,
                showSummary && (React.createElement("li", { className: emotion.cx(listItem, resultPadding) },
                    React.createElement(Summary, { showQueryOverride: false, styles: summaryStyles }))),
                results.map(function (result, idx) { return (React.createElement("li", __assign({}, downshift.getItemProps({
                    key: result.key,
                    item: result,
                    index: idx,
                    role: "option",
                    className: listItem
                })),
                    React.createElement(ResultRenderer, { values: result.values, isHighlighted: downshift.highlightedIndex === idx }))); })))));
        }));
    };
    Results$$1.defaultProps = {
        ResultRenderer: ResultItem$1,
        config: defaultConfig,
        showSummary: false
    };
    return Results$$1;
}(React.Component));
function ResultItem$1(_a) {
    var values = _a.values, isHighlighted = _a.isHighlighted;
    var title = values.title;
    var description = values.description;
    return (React.createElement("div", { className: emotion.cx(resultPadding, isHighlighted && resultHighlighted) },
        React.createElement("h3", { className: titleCSS }, title),
        React.createElement("p", { className: descriptionCSS }, description)));
}
var listItem = emotion.css({
    listStyle: "none",
    paddingLeft: 0,
    marginLeft: 0,
    backgroundColor: "#fff",
    cursor: "auto"
});
var resultPadding = emotion.css({ padding: "0.5em 1em" });
var resultHighlighted = emotion.css({
    backgroundColor: "#eee",
    cursor: "default"
});
var titleCSS = emotion.css({
    fontSize: "1.1em",
    fontWeight: 400,
    lineHeight: 1.1,
    marginBottom: 0,
    marginTop: 0,
    overflow: "hidden",
    textOverflow: "ellipsis",
    whiteSpace: "nowrap"
});
var descriptionCSS = emotion.css({
    color: "#545454",
    fontSize: "0.85em",
    lineHeight: 1.4,
    marginBottom: 4,
    marginTop: 2,
    overflowWrap: "break-word",
    wordWrap: "break-word"
});
var summaryStyles = {
    container: {
        marginBottom: 0
    }
};

var isNotEmptyString = function (a, b) { return (a === "" ? b : a); };
function isNotEmptyArray(a, b) {
    if (!Array.isArray(a) || !Array.isArray(b)) {
        throw new Error("provided arguments must be arrays");
    }
    return a.length === 0 ? b : a;
}
function trimPrefix(str, prefix) {
    if (str === undefined || prefix === "") {
        return "";
    }
    else if (str.startsWith(prefix)) {
        return str.slice(prefix.length);
    }
    else {
        return "";
    }
}

function Suggestions(_a) {
    var downshift = _a.downshift, typedInputValue = _a.typedInputValue, _b = _a.suggestions, suggestions = _b === void 0 ? [] : _b, styles = _a.styles;
    return (React.createElement(Dropdown, { downshift: downshift, className: "sj-input__suggestions", styles: styles && styles.container }, downshift.isOpen &&
        suggestions.length > 0 &&
        suggestions.map(function (suggestion, idx) { return (React.createElement("li", __assign({}, downshift.getItemProps({
            key: suggestion,
            item: suggestion,
            index: idx,
            role: "option",
            className: emotion.cx("sj-input__suggestions__item", downshift.highlightedIndex === idx &&
                "sj-input__suggestions__item--highlighted", emotion.css(suggestionItemStyles(downshift.highlightedIndex === idx)), styles &&
                styles.item &&
                emotion.css(styles.item(downshift.highlightedIndex === idx)))
        })), suggestionText(typedInputValue, suggestion))); })));
}
function suggestionText(inputValue, suggestion) {
    return (React.createElement(React.Fragment, null, inputValue === "" || !suggestion.startsWith(inputValue) ? (suggestion) : (React.createElement(React.Fragment, null,
        inputValue,
        React.createElement("strong", null, trimPrefix(suggestion, inputValue))))));
}
var suggestionItemStyles = function (isFocused) { return ({
    listStyle: "none",
    marginLeft: 0,
    padding: "0.25em 0.5em",
    color: isFocused ? "#222" : "inherit",
    backgroundColor: isFocused ? "#eee" : "inherit",
    cursor: isFocused ? "default" : "auto"
}); };

function Typeahead(_a) {
    var inputValue = _a.inputValue, completion = _a.completion, styles = _a.styles;
    var typeaheadValue = "";
    if (completion) {
        typeaheadValue = trimPrefix(completion, inputValue);
    }
    return (React.createElement("div", { className: emotion.cx("sj-input__typeahead", typeaheadStyles, styles && emotion.css(styles)) }, typeaheadValue));
}
var typeaheadStyles = emotion.css({
    color: "#bebebe",
    display: "inline",
    fontSize: "1em",
    marginLeft: -2,
    position: "relative"
});

/* tslint:disable max-classes-per-file */
var ReturnKeyCode = 13;
var RightArrowKeyCode = 39;
var Input = /** @class */ (function (_super) {
    __extends(Input, _super);
    function Input() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { typedInputValue: "" };
        _this.handleInputOnChange = function (event) {
            // @ts-ignore: value is a member of event.target
            var typedInputValue = event.target.value;
            _this.setState(function (state) { return (__assign({}, state, { typedInputValue: typedInputValue })); });
        };
        _this.inputRef = function (element) {
            _this.input = element;
            if (typeof _this.props.inputRef === "function") {
                _this.props.inputRef(element);
            }
        };
        _this.stateReducer = function (pipelines, items) { return function (state, changes) {
            switch (changes.type) {
                case Downshift.stateChangeTypes.changeInput:
                    if (_this.props.instantSearch) {
                        pipelines.search.search(changes.inputValue || "", false);
                    }
                    else if (_this.props.mode === "typeahead") {
                        pipelines.instant.search(changes.inputValue || "", false);
                    }
                    if (_this.props.dropdownMode !== "results") {
                        if (changes.inputValue === "") {
                            // if we clear the input, reset the selected item back to null
                            return __assign({}, changes, { selectedItem: null });
                        }
                        return __assign({}, changes, { selectedItem: changes.inputValue });
                    }
                    else if (_this.props.dropdownMode === "results") {
                        return __assign({}, changes, { highlightedIndex: null });
                    }
                    return changes;
                case Downshift.stateChangeTypes.keyDownArrowUp:
                    if (state.highlightedIndex === 0) {
                        return __assign({}, changes, { inputValue: _this.state.typedInputValue, selectedItem: _this.state.typedInputValue, highlightedIndex: null });
                    }
                    if (changes.highlightedIndex != null) {
                        var item = (items || [])[changes.highlightedIndex];
                        if (typeof item !== "string") {
                            return changes;
                        }
                        return __assign({}, changes, { inputValue: item, selectedItem: item });
                    }
                    return changes;
                case Downshift.stateChangeTypes.keyDownArrowDown:
                    if (state.highlightedIndex === (items || []).length - 1) {
                        return __assign({}, changes, { inputValue: _this.state.typedInputValue, selectedItem: _this.state.typedInputValue, highlightedIndex: null });
                    }
                    if (changes.highlightedIndex != null) {
                        var item = (items || [])[changes.highlightedIndex];
                        if (typeof item !== "string") {
                            return changes;
                        }
                        return __assign({}, changes, { inputValue: item, selectedItem: item });
                    }
                    return changes;
                case Downshift.stateChangeTypes.clickItem:
                    if (_this.input) {
                        _this.input.blur();
                    }
                    if (_this.props.dropdownMode === "results") {
                        var url = changes.selectedItem !== undefined &&
                            changes.selectedItem.token !== undefined
                            ? changes.selectedItem.token
                            : changes.selectedItem.values.url;
                        window.location.href = url;
                        return state;
                    }
                    pipelines.search.search(changes.inputValue || "", true);
                    return changes;
                case Downshift.stateChangeTypes.keyDownEscape:
                    if (_this.props.dropdownMode === "suggestions") {
                        return __assign({}, changes, { inputValue: _this.state.typedInputValue, selectedItem: _this.state.typedInputValue });
                    }
                    return changes;
                case Downshift.stateChangeTypes.itemMouseEnter:
                    if (_this.props.dropdownMode === "suggestions" &&
                        changes.highlightedIndex != null) {
                        var item = (items || [])[changes.highlightedIndex];
                        if (typeof item !== "string") {
                            return changes;
                        }
                        return __assign({}, changes, { inputValue: item, selectedItem: item });
                    }
                    return changes;
                default:
                    return changes;
            }
        }; };
        return _this;
    }
    Input.prototype.render = function () {
        var _this = this;
        return (React.createElement(Consumer, null, function (pipelines) {
            var items = _this.props.dropdownMode === "suggestions"
                ? isNotEmptyArray(pipelines.search.suggestions, pipelines.instant.suggestions)
                : _this.props.dropdownMode === "results"
                    ? isNotEmptyArray((pipelines.search.response &&
                        pipelines.search.response.getResults()) ||
                        [], (pipelines.instant.response &&
                        pipelines.instant.response.getResults()) ||
                        [])
                    : undefined;
            return (React.createElement(Downshift, { key: "sj-downshift-" + _this.props.defaultValue, defaultInputValue: _this.props.defaultValue, stateReducer: _this.stateReducer(pipelines, items), itemToString: _this.props.dropdownMode === "results"
                    ? function (item) {
                        return (item != undefined && item.values.title) || "";
                    }
                    : undefined }, function (downshift) { return (React.createElement(Inner, __assign({}, downshift.getRootProps({ refKey: "rootRef" }), { mode: _this.props.mode, dropdownMode: _this.props.dropdownMode, ariaLabel: _this.props.ariaLabel, placeholder: _this.props.placeholder, pipelines: pipelines, items: items, downshift: downshift, autoFocus: _this.props.autoFocus, buttonText: _this.props.buttonText, inputRef: _this.inputRef, onKeyDown: _this.props.onKeyDown, onFocus: _this.props.onFocus, onBlur: _this.props.onBlur, onChange: _this.handleInputOnChange, ResultRenderer: _this.props.ResultRenderer, className: _this.props.className, theme: _this.props.theme, styles: _this.props.styles }))); }));
        }));
    };
    Input.defaultProps = {
        mode: "standard",
        dropdownMode: "none",
        ariaLabel: "Search through the site content",
        placeholder: "Search"
    };
    return Input;
}(React.PureComponent));
// connect input to theme
var Input$1 = emotionTheming.withTheme(Input);
var Inner = /** @class */ (function (_super) {
    __extends(Inner, _super);
    function Inner() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { focused: false, typedInputValue: "" };
        _this.rootRef = function (element) {
            _this.root = element;
            if (typeof _this.props.rootRef === "function") {
                _this.props.rootRef(element);
            }
        };
        _this.inputRef = function (element) {
            _this.input = element;
            if (typeof _this.props.inputRef === "function") {
                _this.props.inputRef(element);
            }
        };
        _this.buttonRef = function (element) {
            _this.searchButton = element;
        };
        _this.getButtonWidth = function () {
            if (_this.searchButton) {
                return _this.searchButton.offsetWidth;
            }
            return 0;
        };
        _this.focusInput = function (_) {
            if (_this.input) {
                _this.input.focus();
                _this.setState(function (state) { return (__assign({}, state, { focused: true })); });
            }
        };
        _this.onSearchButtonClick = function (event) {
            event.preventDefault();
            _this.props.pipelines.search.search(_this.props.downshift.selectedItem, true);
        };
        return _this;
    }
    Inner.prototype.componentDidMount = function () {
        if (!this.state.focused) {
            if (this.props.autoFocus && this.input != null) {
                this.input.focus();
            }
        }
    };
    Inner.prototype.render = function () {
        var _this = this;
        var _a = this.props, downshift = _a.downshift, _b = _a.items, items = _b === void 0 ? [] : _b;
        return (React.createElement("div", { className: emotion.cx("sj-input", this.props.className, this.props.styles &&
                this.props.styles.container &&
                emotion.css(this.props.styles.container)) },
            React.createElement("form", { ref: this.rootRef, onClick: this.focusInput, className: emotion.cx("sj-input__input", this.state.focused && "sj-input__input--focused", emotion.css(inputContainerStyles(this.state.focused)), this.props.styles &&
                    this.props.styles.input &&
                    emotion.css(this.props.styles.input(this.state.focused))) },
                React.createElement("div", { role: "search", className: innerContainerStyles(this.getButtonWidth()) },
                    React.createElement(AutosizeInput, __assign({ inputRef: this.inputRef, className: emotion.css(inputResetStyles.container), inputStyle: inputResetStyles.input, minWidth: 5 }, downshift.getInputProps({
                        "aria-label": this.props.ariaLabel,
                        placeholder: this.props.placeholder,
                        autoComplete: "off",
                        spellCheck: false,
                        autoCapitalize: "off",
                        autoCorrect: "off",
                        maxLength: 2048,
                        onFocus: function (event) {
                            downshift.openMenu();
                            _this.setState(function (state) { return (__assign({}, state, { focused: true })); });
                            if (typeof _this.props.onFocus === "function") {
                                _this.props.onFocus(event);
                            }
                        },
                        onBlur: function (event) {
                            downshift.closeMenu();
                            downshift.setState({ highlightedIndex: null });
                            _this.setState(function (state) { return (__assign({}, state, { focused: false })); });
                            if (typeof _this.props.onBlur === "function") {
                                _this.props.onBlur(event);
                            }
                        },
                        onChange: function (event) {
                            _this.props.onChange(event);
                            // @ts-ignore: value is a member of event.target
                            var typedInputValue = event.target.value;
                            _this.setState(function (state) { return (__assign({}, state, { typedInputValue: typedInputValue })); });
                        },
                        onKeyDown: function (event) {
                            if (event.keyCode === ReturnKeyCode) {
                                event.preventDefault();
                                // @ts-ignore: Prevent Downshift's default 'Enter' behavior.
                                event.nativeEvent.preventDownshiftDefault = true;
                                if (_this.input) {
                                    _this.input.blur();
                                }
                                var item = downshift.selectedItem;
                                if (_this.props.dropdownMode === "results") {
                                    item = (_this.props.items || [])[downshift.highlightedIndex || -1];
                                    if (item === undefined) {
                                        return;
                                    }
                                    var url = Object.getOwnPropertyNames(item.token).length > 0
                                        ? item.token
                                        : item.values.url;
                                    window.location.href = url;
                                    return;
                                }
                                if (downshift.highlightedIndex !== null) {
                                    downshift.selectHighlightedItem();
                                    item = (_this.props.items || [])[downshift.highlightedIndex];
                                }
                                _this.props.pipelines.search.search(item, true);
                            }
                            if (event.keyCode === RightArrowKeyCode) {
                                if (_this.props.dropdownMode !== "results") {
                                    if (downshift.highlightedIndex !== null &&
                                        // @ts-ignore: selectionStart is a member of event.target
                                        event.target.selectionStart ===
                                            (downshift.inputValue || "").length) {
                                        // if the user presses the right arrow key and there is
                                        // a highlighted item, select the currently highlighted item
                                        // Though make sure we only select the item if we are at the
                                        // end of the input element
                                        downshift.selectHighlightedItem();
                                    }
                                    else if (downshift.highlightedIndex === null &&
                                        // @ts-ignore: selectionStart is a member of event.target
                                        event.target.selectionStart ===
                                            (downshift.inputValue || "").length) {
                                        downshift.selectItem(isNotEmptyString(_this.props.pipelines.search.completion, _this.props.pipelines.instant.completion));
                                    }
                                }
                                else {
                                    _this.props.downshift.setState({
                                        inputValue: isNotEmptyString(_this.props.pipelines.search.completion, _this.props.pipelines.instant.completion)
                                    });
                                }
                            }
                            if (typeof _this.props.onKeyDown === "function") {
                                _this.props.onKeyDown(event);
                            }
                        }
                    }), { "aria-labelledby": undefined })),
                    this.props.mode === "typeahead" && (React.createElement(Typeahead, { inputValue: downshift.inputValue || "", completion: this.props.dropdownMode !== "suggestions"
                            ? isNotEmptyString(this.props.pipelines.search.completion, this.props.pipelines.instant.completion)
                            : undefined, styles: this.props.styles && this.props.styles.typeahead }))),
                React.createElement("button", { ref: this.buttonRef, className: emotion.cx("sj-input__button", buttonStyles(this.props.theme &&
                        this.props.theme.colors &&
                        this.props.theme.colors.brand &&
                        this.props.theme.colors.brand.primary), this.props.styles &&
                        this.props.styles.button &&
                        emotion.css(this.props.styles.button)), onClick: this.onSearchButtonClick, "aria-label": "Search", title: "Search", value: "Search" },
                    React.createElement(SearchIcon, null),
                    this.props.buttonText && (React.createElement("span", { className: buttonTextStyles }, this.props.buttonText)))),
            this.props.dropdownMode === "suggestions" && (React.createElement(Suggestions, { downshift: downshift, typedInputValue: this.state.typedInputValue, suggestions: items, styles: this.props.styles && this.props.styles.suggestions })),
            this.props.dropdownMode === "results" && (React.createElement(Results$1, { downshift: downshift, ResultRenderer: this.props.ResultRenderer }))));
    };
    return Inner;
}(React.Component));
var SearchIcon = function (props) { return (React.createElement("svg", __assign({ viewBox: "0 0 18 18", fill: "none", width: "1em", height: "1em" }, props),
    React.createElement("path", { d: "M12.5 11h-.79l-.28-.27A6.471 6.471 0 0 0 13 6.5 6.5 6.5 0 1 0 6.5 13c1.61 0 3.09-.59 4.23-1.57l.27.28v.79l5 4.99L17.49 16l-4.99-5zm-6 0C4.01 11 2 8.99 2 6.5S4.01 2 6.5 2 11 4.01 11 6.5 8.99 11 6.5 11z", fill: "currentcolor" }))); };
var inputContainerStyles = function (isFocused) { return ({
    display: "flex",
    justifyContent: "space-between",
    alignItems: "center",
    padding: "3px 9px",
    "&:hover": {
        boxShadow: "0 3px 8px 0 rgba(0,0,0,0.16), 0 0 0 1px rgba(0,0,0,0.08)"
    },
    borderRadius: 2,
    boxShadow: "0 2px 2px 0 rgba(0,0,0,0.16), 0 0 0 1px rgba(0,0,0,0.08)",
    transition: "box-shadow 200ms cubic-bezier(0.4, 0.0, 0.2, 1)",
    outline: isFocused
        ? ["#5E9ED6 auto 5px", "-webkit-focus-ring-color auto 5px"]
        : 0
}); };
var innerContainerStyles = function (buttonWidth) {
    return emotion.css({
        display: "flex",
        flex: "1 1 auto",
        flexDirection: "row",
        justifyContent: "start",
        alignItems: "baseline",
        maxWidth: "calc(100% - " + buttonWidth + "px)"
    });
};
var inputResetStyles = {
    container: {
        "& > div[style]": {
            display: "inline",
            // fixes sizing bug in firefox v60.0.2
            overflow: "unset !important"
        },
        "&[style]": {
            display: ["inline-block !important", "-moz-inline-stack !important"]
        },
        background: "none",
        overflow: "auto"
    },
    input: {
        background: 0,
        border: 0,
        color: "inherit",
        fontFamily: "inherit",
        fontSize: "inherit",
        outline: 0,
        padding: 0,
        textRendering: "optimizeLegibility"
    }
};
var buttonStyles = function (primary) {
    return emotion.css({
        boxSizing: "content-box",
        padding: 8,
        border: 0,
        fontSize: 18,
        backgroundColor: "transparent",
        display: "flex",
        alignItems: "center",
        cursor: "pointer",
        color: "#737373",
        "&:hover": {
            color: primary != undefined ? primary : "#222"
        }
    });
};
var buttonTextStyles = emotion.css({
    paddingLeft: 8
});

var Response$1 = function (_a) {
    var children = _a.children;
    return (React.createElement(Consumer$1, null, function (_a) {
        var response = _a.search.response;
        if (response === null || response === undefined || response.isEmpty()) {
            return null;
        }
        return children;
    }));
};

var windowSize = 5;
var minPage = 1;
var pageNumbers = function (page, totalPages) {
    var pages = [page];
    for (var i = 1; i < windowSize; i++) {
        pages.unshift(page - i);
        pages.push(page + i);
    }
    // filter out negative values
    pages = pages.filter(function (v) { return v > 0; });
    return windowFn(page, pages, minPage, totalPages, windowSize);
};
var windowFn = function (center, arr, min, max, size) {
    var centerIdx = arr.indexOf(center);
    var maxIdx = arr.indexOf(max);
    if (max < size) {
        size = max;
    }
    var start = centerIdx - 2;
    var end = centerIdx + 3;
    // If the center of the window is less than half,
    // just show the first window
    if (centerIdx <= Math.ceil(size / center)) {
        start = 0;
        end = size;
        return arr.slice(start > 0 ? start : 0, end);
    }
    if (maxIdx !== -1 && maxIdx - centerIdx < 2) {
        start = maxIdx - 4;
        end = maxIdx + 1;
        return arr.slice(start > 0 ? start : 0, end);
    }
    return arr.slice(start > 0 ? start : 0, end);
};

var LeftChevron = function (props) { return (React.createElement("svg", __assign({ viewBox: "0 0 8 12", fill: "none", width: "1em", height: "1em" }, props),
    React.createElement("path", { d: "M7.41 1.41L6 0 0 6l6 6 1.41-1.41L2.83 6l4.58-4.59z", fill: "currentcolor" }))); };
var RightChevron = function (props) { return (React.createElement("svg", __assign({ viewBox: "0 0 8 12", fill: "none", width: "1em", height: "1em" }, props),
    React.createElement("path", { d: "M2 0L.59 1.41 5.17 6 .59 10.59 2 12l6-6-6-6z", fill: "currentcolor" }))); };

// @ts-ignore: module missing definitions
var Container$2 = styled("nav")({
    display: "flex",
    justifyContent: "center",
    alignItems: "center"
    // marginBottom: "1em"
}, override);
var PageNumber = styled("a")({
    cursor: "pointer",
    display: ["inline-block", "-moz-inline-stack"],
    fontWeight: "bold",
    padding: 10,
    userSelect: "none",
    width: 44,
    height: 44,
    textAlign: "center"
}, function (_a) {
    var curr = _a.isCurrent, theme = _a.theme;
    // @ts-ignore: idx
    var themeColor = idx_1(theme, function (_) { return _.colors.brand.primary; });
    var textColor = chroma.contrast("#fff", themeColor || "#333") > 4.5 ? "#fff" : "#000";
    return {
        borderRadius: 5,
        backgroundColor: curr ? themeColor || "#333" : "inherit",
        color: curr ? textColor : "#585858"
    };
}, override);
var PageButton = styled("button")({
    backgroundColor: "transparent",
    border: "none",
    cursor: "pointer",
    display: ["inline-block", "-moz-inline-stack"],
    fontSize: "1em",
    fontWeight: "bold",
    padding: 10,
    userSelect: "none",
    width: 44,
    height: 44,
    textAlign: "center",
    lineHeight: 0
}, function (props) { return ({ color: props.isDisabled ? "#aaa" : "#777" }); }, override);

var Paginator = /** @class */ (function (_super) {
    __extends(Paginator, _super);
    function Paginator() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.prevPage = function (paginate, page) { return function () {
            if (page === 1) {
                return;
            }
            _this.setPage(paginate, page - 1)();
        }; };
        _this.nextPage = function (paginate, page, totalPages) { return function () {
            if (page === totalPages) {
                return;
            }
            _this.setPage(paginate, page + 1)();
        }; };
        _this.setPage = function (paginate, page) { return function () {
            window.scrollTo(0, 0);
            paginate(page);
        }; };
        return _this;
    }
    Paginator.prototype.render = function () {
        var _this = this;
        var _a = this.props.styles, styles = _a === void 0 ? {} : _a;
        return (React.createElement(Consumer$1, null, function (_a) {
            var _b = _a.search, response = _b.response, config = _b.config, paginate = _a.paginate;
            if (response === null ||
                response === undefined ||
                response.isEmpty() ||
                response.isError()) {
                return null;
            }
            var queryValues = response.getQueryValues();
            if (queryValues === undefined) {
                return;
            }
            var page = queryValues.get(config.pageParam)
                ? parseInt(queryValues.get(config.pageParam), 10)
                : 1;
            var resultsPerPage = queryValues.get(config.resultsPerPageParam)
                ? parseInt(queryValues.get(config.resultsPerPageParam), 10)
                : 10;
            var totalResults = response.getTotalResults();
            if (totalResults === undefined) {
                return;
            }
            if (totalResults <= resultsPerPage) {
                return null;
            }
            var totalPages = Math.ceil(totalResults / resultsPerPage);
            if (totalPages === 0) {
                return null;
            }
            return (React.createElement(Container$2, { className: emotion.cx("sj-paginator", _this.props.className), "aria-label": "Pagination Navigation", styles: styles.container },
                page !== 1 && (React.createElement(PageButton, { isDisabled: page === 1, onClick: _this.prevPage(paginate, page), "aria-label": "Goto Previous Page", className: "sj-paginator__page-button", styles: styles.controls },
                    React.createElement(LeftChevron, null))),
                React.createElement("ul", { className: emotion.css({
                        display: "inline-flex",
                        listStyle: "none",
                        margin: 0,
                        padding: 0
                    }) }, pageNumbers(page, totalPages).map(function (pageNumber) { return (React.createElement("li", { key: pageNumber },
                    React.createElement(PageNumber, { className: emotion.cx("sj-paginator__page-number", page === pageNumber &&
                            "sj-paginator__page-number--current", styles.number &&
                            emotion.css(styles.number(page === pageNumber))), isCurrent: page === pageNumber, onClick: _this.setPage(paginate, pageNumber), "aria-label": page === pageNumber
                            ? "Current Page, Page " + pageNumber
                            : "Page " + pageNumber, "aria-current": page === pageNumber ? true : undefined }, pageNumber))); })),
                page !== totalPages && (React.createElement(PageButton, { isDisabled: page === totalPages, onClick: _this.nextPage(paginate, page, totalPages), "aria-label": "Goto Next Page", className: "sj-paginator__page-button", styles: styles.controls },
                    React.createElement(RightChevron, null)))));
        }));
    };
    return Paginator;
}(React.Component));

var Container$3 = styled("div")({
    borderBottom: "1px solid #ebebeb",
    color: "#777",
    marginBottom: "1em",
    width: "100%"
}, override);
var TabsContainer = styled("div")({
    overflow: "auto",
    whiteSpace: "nowrap"
});

var Container$4 = styled("div")({
    boxSizing: "border-box",
    cursor: "pointer",
    display: ["inline-block", "-moz-inline-stack"],
    fontSize: 16,
    margin: 0,
    padding: ".45em .9em .5em",
    userSelect: "none"
}, function (_a) {
    var theme = _a.theme, isSelected = _a.isSelected;
    return isSelected
        ? {
            borderBottom: "3px solid",
            // @ts-ignore: idx
            color: idx_1(theme, function (_) { return _.colors.brand.primary; }) || "#333"
        }
        : {
            "&:hover": {
                // @ts-ignore: idx
                color: idx_1(theme, function (_) { return _.colors.brand.primary; }) || "#333"
            },
            borderBottom: "3px solid transparent"
        };
}, override);

var Tab = function (_a) {
    var title = _a.title, _b = _a.styles, styles = _b === void 0 ? {} : _b;
    return (React.createElement(FilterConsumer, null, function (_a) {
        var selected = _a.selected, set = _a.set;
        var isSelected = selected.includes(title);
        return (React.createElement(Container$4, { className: emotion.cx("sj-tabs__tab", isSelected && "sj-tabs__tab--selected"), isSelected: isSelected, onClick: setFilter(set, title, !isSelected), style: styles }, title));
    }));
};
var setFilter = function (set, name, value) { return function () {
    if (!value) {
        // make tab act like radio button
        return;
    }
    set(name, value);
}; };

var Tabs = function (_a) {
    var filter = _a.filter, tabs = _a.tabs, _b = _a.styles, styles = _b === void 0 ? {} : _b;
    return (React.createElement(FilterProvider, { filter: filter },
        React.createElement(Container$3, { className: "sj-tabs", styles: styles.container },
            React.createElement(TabsContainer, null, tabs.map(function (tab) { return (React.createElement(Tab, { key: tab.name, title: tab.display, styles: styles.tab })); })))));
};

// tslint:disable
var styles = {
    container: function (base, _a) {
        var isFocused = _a.isFocused;
        return __assign({}, base, { borderBottom: "1px solid", borderBottomColor: isFocused ? "currentcolor" : "#ccc", display: ["inline-block", "-moz-inline-stack"], padding: "0.25em 0", width: "auto" });
    },
    control: function (base) {
        return __assign({}, base, { backgroundColor: "transparent", border: "none", borderRadius: 0, boxShadow: null, color: "#666", display: "inline-flex", fontSize: "0.8em", minHeight: 0, minWidth: "120px" });
    },
    option: function (base, _a) {
        var isDisabled = _a.isDisabled, isFocused = _a.isFocused, isSelected = _a.isSelected;
        return __assign({}, base, { backgroundColor: isDisabled
                ? null
                : isSelected || isFocused
                    ? "#eaeaea"
                    : null, color: "#666" });
    },
    menu: function (base) { return (__assign({}, base, { borderRadius: 0, fontSize: "0.8em", width: "100%", zIndex: 1000 })); },
    menuList: function (base) { return (__assign({}, base, { padding: 0 })); },
    indicatorSeparator: function () { return ({
        display: "none"
    }); },
    dropdownIndicator: function (base) { return (__assign({}, base, { padding: "0 0 0 .5em" })); },
    clearIndicator: function (base) { return (__assign({}, base, { padding: "0 0 0 .5em" })); },
    singleValue: function () { return ({
        fontWeight: "bold",
        padding: 0
    }); },
    placeholder: function () { return ({
        padding: 0
    }); },
    input: function () { return ({
        padding: 0
    }); },
    valueContainer: function (base) { return (__assign({}, base, { padding: 0 })); }
};

var Select = function (_a) {
    var filter = _a.filter;
    return (React.createElement(FilterProvider, { filter: filter },
        React.createElement(FilterConsumer, null, function (_a) {
            var options = _a.options, selected = _a.selected, set = _a.set;
            var opts = mapOptions(options);
            var defaultValue = opts.filter(function (x) { return x.label === selected[0]; }).pop();
            return (React.createElement(ReactSelect, { key: "1" + defaultValue, defaultValue: defaultValue, isClearable: true, options: mapOptions(options), onChange: handleChange(selected, set), styles: styles }));
        })));
};
var mapOptions = memoize(function (options) {
    return Object.entries(options).map(function (_a) {
        var label = _a[0], value = _a[1];
        return ({
            label: label,
            value: value
        });
    });
});
var ActionSelect = "select-option";
var ActionPop = "pop-value";
var ActionClear = "clear";
var handleChange = function (selected, set) { return function (value, _a) {
    var action = _a.action;
    switch (action) {
        case ActionSelect:
            set(value.label, true);
            break;
        case ActionPop:
            set(selected[0], false);
            break;
        case ActionClear:
            selected.forEach(function (option) {
                set(option, false);
            });
            break;
        default:
            break;
    }
}; };

var Container$5 = styled("div")({
    "&:hover": {
        color: "#333"
    },
    cursor: "pointer",
    display: ["inline-block", "-moz-inline-stack"]
}, override);
var HiddenInput = styled("input")({
    border: 0,
    clip: "rect(1px, 1px, 1px, 1px)",
    height: 1,
    opacity: 0,
    overflow: "hidden",
    padding: 0,
    position: "absolute",
    whiteSpace: "nowrap",
    width: 1
});
var NativeInput = styled("input")({
    cursor: "pointer"
});

var Checkbox = /** @class */ (function (_super) {
    __extends(Checkbox, _super);
    function Checkbox() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onClick = function (name, isSelected, set) { return function () { return set(name, !isSelected); }; };
        return _this;
    }
    Checkbox.prototype.render = function () {
        var _this = this;
        var _a = this.props, name = _a.name, _b = _a.CheckboxRenderer, CheckboxRenderer = _b === void 0 ? NativeCheckbox : _b;
        return (React.createElement(FilterConsumer, null, function (_a) {
            var selected = _a.selected, set = _a.set;
            var isSelected = selected.includes(name);
            return (React.createElement(Container$5, { isSelected: isSelected, onClick: _this.onClick(name, isSelected, set) },
                React.createElement(HiddenInput, { type: "checkbox", value: name, checked: isSelected }),
                React.createElement(CheckboxRenderer, { isChecked: isSelected })));
        }));
    };
    return Checkbox;
}(React.Component));
var NativeCheckbox = function (_a) {
    var isChecked = _a.isChecked;
    return (React.createElement(NativeInput, { type: "checkbox", checked: isChecked }));
};

var Container$6 = styled(Container$5)({});

var Radio = /** @class */ (function (_super) {
    __extends(Radio, _super);
    function Radio() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onClick = function (name, isSelected, set) { return function () {
            if (isSelected) {
                return;
            }
            set(name, !isSelected);
        }; };
        return _this;
    }
    Radio.prototype.render = function () {
        var _this = this;
        var _a = this.props, name = _a.name, _b = _a.RadioRenderer, RadioRenderer = _b === void 0 ? NativeRadio : _b;
        return (React.createElement(FilterConsumer, null, function (_a) {
            var selected = _a.selected, set = _a.set;
            var isSelected = selected.includes(name);
            return (React.createElement(Container$6, { isSelected: isSelected, onClick: _this.onClick(name, isSelected, set) },
                React.createElement(HiddenInput, { type: "radio", value: name, checked: isSelected }),
                React.createElement(RadioRenderer, { isChecked: isSelected })));
        }));
    };
    return Radio;
}(React.Component));
var NativeRadio = function (_a) {
    var isChecked = _a.isChecked;
    return (React.createElement(NativeInput, { type: "radio", checked: isChecked }));
};

var OverlayZIndex = 10000000;
var BlurContainer = styled("div")({
    position: "absolute",
    right: 0,
    top: 0,
    left: 0,
    bottom: 0,
    zIndex: OverlayZIndex,
    backgroundColor: "rgba(0,0,0,0.5)",
    display: "flex",
    justifyContent: "center",
    alignItems: "center"
});
var Container$7 = styled("div")({
    backgroundColor: "#fff",
    boxSizing: "border-box",
    height: "80%",
    padding: "1em",
    width: "80%"
});

var Overlay = /** @class */ (function (_super) {
    __extends(Overlay, _super);
    function Overlay() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onClick = function (event) {
            var target = event.target;
            // @ts-ignore: nodeName is a property of event.target
            if (target.nodeName !== "DIV") {
                return;
            }
            // @ts-ignore: className is a property of event.target
            if ((target.className || "").split(" ").includes("sj-overlay")) {
                _this.props.onOuterClick(event);
            }
        };
        return _this;
    }
    Overlay.prototype.render = function () {
        var isActive = this.props.isActive;
        if (!isActive) {
            return null;
        }
        var children = this.props.children;
        return (React.createElement(BlurContainer, { className: "sj-overlay__shade", onClick: this.onClick },
            React.createElement(Container$7, { className: "sj-overlay__container" }, children)));
    };
    return Overlay;
}(React.Component));

exports.Pipeline = Pipeline;
exports.Values = Values;
exports.Filter = Filter;
exports.CombineFilters = CombineFilters;
exports.ClickTracking = ClickTracking;
exports.NoTracking = NoTracking;
exports.GoogleAnalytics = GoogleAnalytics;
exports.DebugAnalytics = DebugAnalytics;
exports.Consumer = Consumer$1;
exports.Provider = Provider$1;
exports.FilterProvider = FilterProvider;
exports.Input = Input$1;
exports.Response = Response$1;
exports.ResultsContainer = ResultsContainer;
exports.Results = Results;
exports.TokenLink = TokenLink;
exports.Summary = Summary;
exports.Paginator = Paginator;
exports.Tabs = Tabs;
exports.Select = Select;
exports.Checkbox = Checkbox;
exports.Radio = Radio;
exports.Overlay = Overlay;
exports.EVENT_SELECTION_UPDATED = EVENT_SELECTION_UPDATED;
exports.EVENT_OPTIONS_UPDATED = EVENT_OPTIONS_UPDATED;
exports.EVENT_SEARCH_SENT = EVENT_SEARCH_SENT;
exports.EVENT_RESPONSE_UPDATED = EVENT_RESPONSE_UPDATED;
exports.EVENT_RESULT_CLICKED = EVENT_RESULT_CLICKED;
exports.EVENT_ANALYTICS_PAGE_CLOSED = EVENT_ANALYTICS_PAGE_CLOSED;
exports.EVENT_ANALYTICS_BODY_RESET = EVENT_ANALYTICS_BODY_RESET;
exports.EVENT_ANALYTICS_RESULT_CLICKED = EVENT_ANALYTICS_RESULT_CLICKED;
exports.EVENT_VALUES_UPDATED = EVENT_VALUES_UPDATED;
exports.EVENT_TRACKING_RESET = EVENT_TRACKING_RESET;
exports.i18n = i18n;
